﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;
using Antlr.Runtime;

namespace ManagedMyC
{
    class MyCScanner : IScanner
    {
        public MyCScanner( MyCLanguageService languageService, IVsTextLines buffer )
        {
            LanguageService = languageService;
            Buffer = buffer;
        }

        MyCLanguageService LanguageService
        {
            get;
            set;
        }
        IVsTextLines Buffer
        {
            get;
            set;
        }
        string Source
        {
            get;
            set;
        }
        int Offset
        {
            get;
            set;
        }

        MyCLexer _lexer;
        ScannerState _state;

        internal struct ScannerState
        {
            int _state;

            public ScannerState( int state )
            {
                _state = state;
            }

            public int State
            {
                get
                {
                    return _state;
                }
                set
                {
                    _state = value;
                }
            }

            [Flags]
            enum Flags
            {
                InBlockComment = 0x1 << 0,
            }

            bool GetValue( Flags flag )
            {
                return ( State & (int)flag ) == (int)flag;
            }
            void SetValue( Flags flag, bool value )
            {
                if ( value )
                    State |= (int)flag;
                else
                    State &= ~(int)flag;
            }

            public bool InBlockComment
            {
                get
                {
                    return GetValue( Flags.InBlockComment );
                }
                set
                {
                    SetValue( Flags.InBlockComment, value );
                }
            }
        }

        #region IScanner Members

        public bool ScanTokenAndProvideInfoAboutIt( TokenInfo tokenInfo, ref int state )
        {
            if ( Offset >= Source.Length )
                return false;

            _state.State = state;
            string token_text = string.Empty;

            if ( _lexer == null )
            {
                string text = Source.Substring( Offset );
                ANTLRStringStream input = new ANTLRStringStream( text );
                _lexer = new MyCLexer( input );
                _lexer.InBlockComment = _state.InBlockComment;
            }

            if ( _lexer != null )
            {
                IToken token = _lexer.NextToken();
                token_text = token.Text;
                tokenInfo.Token = token.Type;

                switch ( token.Type )
                {
                case MyCLexer.KWBREAK:
                case MyCLexer.KWCONTINUE:
                case MyCLexer.KWELSE:
                case MyCLexer.KWEXTERN:
                case MyCLexer.KWFOR:
                case MyCLexer.KWIF:
                case MyCLexer.KWINT:
                case MyCLexer.KWRETURN:
                case MyCLexer.KWSTATIC:
                case MyCLexer.KWVOID:
                case MyCLexer.KWWHILE:
                    tokenInfo.Type = TokenType.Keyword;
                    tokenInfo.Color = TokenColor.Keyword;
                    tokenInfo.Trigger = TokenTriggers.None;
                    break;

                case MyCLexer.NUMBER:
                    tokenInfo.Type = TokenType.Literal;
                    tokenInfo.Color = TokenColor.Number;
                    tokenInfo.Trigger = TokenTriggers.None;
                    break;

                case MyCLexer.LPAREN:
                case MyCLexer.RPAREN:
                case MyCLexer.LBRACE:
                case MyCLexer.RBRACE:
                    tokenInfo.Type = TokenType.Delimiter;
                    tokenInfo.Color = TokenColor.Text;
                    tokenInfo.Trigger = TokenTriggers.MatchBraces;
                    break;

                case MyCLexer.COMMENT:
                    tokenInfo.Type = TokenType.Comment;
                    tokenInfo.Color = TokenColor.Comment;
                    tokenInfo.Trigger = TokenTriggers.None;
                    break;

                default:
                    tokenInfo.Type = TokenType.Text;
                    tokenInfo.Color = TokenColor.Text;
                    tokenInfo.Trigger = TokenTriggers.None;
                    break;
                }

                _state.InBlockComment = _lexer.InBlockComment;
            }

            // we reached the end of the line or an unrecognizable token and need to stop parsing this line anyway.
            if ( string.IsNullOrEmpty( token_text ) )
                return false;

            // set the TokenInfo fields to match the location/length of the current token
            tokenInfo.StartIndex = Offset;
            Offset += token_text.Length;
            tokenInfo.EndIndex = Offset - 1;

            state = _state.State;
            return true;
        }

        public void SetSource( string source, int offset )
        {
            if ( offset != Offset || source != Source )
            {
                Source = source;
                Offset = offset;
                _lexer = null;
            }
        }

        #endregion

    }
}
