﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Package;
using Microsoft.VisualStudio.TextManager.Interop;

namespace ManagedMyC
{
    public class MyCSource : Source
    {
        IList<DropDownMember> _types;
        IList<DropDownMember> _members;

        public MyCSource( MyCLanguageService languageService, IVsTextLines textLines, Colorizer colorizer )
            : base( languageService, textLines, colorizer )
        {
        }

        public object ParseResult
        {
            get;
            set;
        }

        public IList<TextSpan[]> Braces
        {
            get;
            set;
        }
        public bool TypesUpdated
        {
            get;
            set;
        }
        public bool MembersUpdated
        {
            get;
            set;
        }
        public IList<DropDownMember> Types
        {
            get
            {
                return _types;
            }
            set
            {
                _types = value;
                TypesUpdated = true;
            }
        }
        public IList<DropDownMember> Members
        {
            get
            {
                return _members;
            }
            set
            {
                _members = value;
                MembersUpdated = true;
            }
        }

        public override CommentInfo GetCommentFormat()
        {
            return new CommentInfo()
            {
                BlockStart = "/*",
                BlockEnd = "*/",
                UseLineComments = false
            };
        }

        #region Type & member dropdowns support
        /// <summary>
        /// Gets the members of the current Source to show in the Types list of the
        /// TypeAndMemberDropdownBars.
        /// </summary>
        /// <returns>An enumeration of items to show in the Types dropdown.</returns>
        public virtual IEnumerable<DropDownMember> GetDropDownTypes()
        {
            if ( Types == null )
                return null;

            return ( from type in Types
                     orderby type.Label
                     select type )
                   .ToArray();
        }
        /// <summary>
        /// Gets the members of the selected type to show in the Members list of the
        /// TypeAndMemberDropdownBars.
        /// </summary>
        /// <param name="type">The item currently selected in the Types dropdown.</param>
        /// <param name="excludeSpans">A list of TextSpans that are used to filter the list of members.
        /// Typically, these spans are sub-spans of the selected type representing nested types, where
        /// members within those spans are actually members of a nested type ant should not be shown.</param>
        /// <returns>An enumeration of items to show in the Members dropdown.</returns>
        public virtual IEnumerable<DropDownMember> GetDropDownMembers( DropDownMember type, IEnumerable<TextSpan> excludeSpans )
        {
            if ( Members == null )
                return null;

            var members = ( from member in Members.ToArray()
                            where (type == null) || TextSpanHelper.IsEmbedded( member.Span, type.Span )
                            where !excludeSpans.Any( span => TextSpanHelper.IsEmbedded( member.Span, span ) )
                            orderby member.Label
                            select member )
                          .ToArray();

            return members;
        }

        /// <summary>
        /// Gets the default item to show in the Types dropdown of the TypeAndMemberDropdownBars.
        /// This function is called when the cursor is not in the span of any type in the list.
        /// </summary>
        /// <param name="types">The types shown in the Types dropdown.</param>
        /// <returns>The index of the type that should be visible in the Types dropdown.</returns>
        public virtual int GetDefaultType( IEnumerable<DropDownMember> types )
        {
            return ( types.FirstOrDefault() == null ) ? -1 : 0;
        }
        /// <summary>
        /// Gets the default item to show in the Members dropdown of the TypeAndMemberDropdownBars.
        /// This function is called when the cursor is not inside the span of any members of the
        /// selected type.
        /// </summary>
        /// <param name="selectedType">The item currently selected in the Types dropdown.</param>
        /// <param name="members">The members of the selected type shown in the Members dropdown.</param>
        /// <returns>The index of the member that should be visible in the Members dropdown.</returns>
        public virtual int GetDefaultMember( DropDownMember selectedType, IEnumerable<DropDownMember> members )
        {
            return ( members.FirstOrDefault() == null ) ? -1 : 0;
        }
        #endregion
    }
}
